import React, { useState, useEffect } from 'react';
import { TableBody, TableRow, TableCell } from '@mui/material';
import { StyledTable, StyledTableContainer, StyledEmptyRow } from './NsTable.styled';
import type { NsTableProps, Item, OrderType } from './types.d';
import { stableSort, getComparator } from './helper';
import NsTableHeader from './NsTableHeader';
import NsCheckBox from '../checkbox/NsCheckbox';
import NsTablePagination from './NsTablePagination';

export default function NsTable({
    headers = [],
    items = [],
    rows,
    isShowSelector = true,
    isPagination = false,
    rowsPerPageOptions,
    itemKey,
}: NsTableProps) {
    const [order, setOrder] = useState<OrderType>('asc');
    const [orderBy, setOrderBy] = useState<keyof Item>(itemKey);
    const [selected, setSelected] = useState<readonly (string | number)[]>([]);
    const [page, setPage] = useState(0);
    const [rowsPerPage, setRowsPerPage] = useState(rows);
    const [itemKeyList, setItemKeyList] = useState<string[]>([]);

    useEffect(() => {
        if (items.length) setItemKeyList(Object.keys(items[0]));
    }, [items]);

    useEffect(() => {
        setRowsPerPage(rows);
        setPage(0);
    }, [rows]);

    const handleRequestSort = (event: React.MouseEvent<unknown>, property: keyof Item) => {
        const isAsc = orderBy === property && order === 'asc';
        setOrder(isAsc ? 'desc' : 'asc');
        setOrderBy(property);
    };

    const handleSelectAllClick = (event: React.ChangeEvent<HTMLInputElement>) => {
        if (event.target.checked) {
            const newSelected = items.map((n) => n[itemKey as string]);
            setSelected(newSelected);
            return;
        }
        setSelected([]);
    };

    const handleClick = (event: React.MouseEvent<unknown>, name: string | number) => {
        const selectedIndex = selected.indexOf(name);
        let newSelected: readonly (string | number)[] = [];

        if (selectedIndex === -1) {
            newSelected = newSelected.concat(selected, name);
        } else if (selectedIndex === 0) {
            newSelected = newSelected.concat(selected.slice(1));
        } else if (selectedIndex === selected.length - 1) {
            newSelected = newSelected.concat(selected.slice(0, -1));
        } else if (selectedIndex > 0) {
            newSelected = newSelected.concat(selected.slice(0, selectedIndex), selected.slice(selectedIndex + 1));
        }

        setSelected(newSelected);
    };

    const handleChangePage = (event: unknown, newPage: number) => {
        setPage(newPage);
    };

    const handleChangeRowsPerPage = (event: React.ChangeEvent<HTMLInputElement>) => {
        setRowsPerPage(parseInt(event.target.value, 10));
        setPage(0);
    };

    const isSelected = (name: string | number) => selected.indexOf(name) !== -1;

    const emptyRows = page > 0 ? Math.max(0, (1 + page) * rowsPerPage - items.length) : 0;

    const visibleRows = React.useMemo(
        () =>
            stableSort(items, getComparator(order, orderBy)).slice(
                page * rowsPerPage,
                page * rowsPerPage + rowsPerPage
            ),
        [order, orderBy, page, rowsPerPage, items]
    );

    return (
        <div>
            <StyledTableContainer>
                <StyledTable>
                    <NsTableHeader
                        headCells={headers}
                        numSelected={selected.length}
                        order={order}
                        orderBy={orderBy as string}
                        onSelectAllClick={handleSelectAllClick}
                        onRequestSort={handleRequestSort}
                        rowCount={items.length}
                        isShowSelector={isShowSelector}
                    />
                    <TableBody>
                        {visibleRows.map((row, index) => {
                            const isItemSelected = isSelected(row[itemKey]);
                            const labelId = `table-checkbox-${index}`;
                            const tableRowTestId = `table-row-${index}`;
                            return (
                                <TableRow
                                    hover
                                    onClick={(event) => handleClick(event, row[itemKey])}
                                    role="checkbox"
                                    aria-checked={isItemSelected}
                                    tabIndex={-1}
                                    key={row[itemKey]}
                                    selected={isItemSelected}
                                    sx={{ cursor: 'pointer' }}
                                    data-testid={tableRowTestId}
                                >
                                    {isShowSelector ? (
                                        <TableCell padding="checkbox">
                                            <NsCheckBox
                                                checked={isItemSelected}
                                                testid={labelId}
                                                name={labelId}
                                                label=""
                                                className="ml-0 !important"
                                            />
                                        </TableCell>
                                    ) : null}
                                    {itemKeyList.map((id, index) => {
                                        return (
                                            <TableCell align="left" key={index}>
                                                {row[id]}
                                            </TableCell>
                                        );
                                    })}
                                </TableRow>
                            );
                        })}
                        {emptyRows > 0 ? (
                            <StyledEmptyRow emptyRows={emptyRows}>
                                <TableCell colSpan={itemKeyList.length} />
                            </StyledEmptyRow>
                        ) : null}
                    </TableBody>
                </StyledTable>
            </StyledTableContainer>
            {isPagination ? (
                <NsTablePagination
                    rowsPerPageOptions={rowsPerPageOptions}
                    count={items.length}
                    rowsPerPage={rowsPerPage}
                    page={page}
                    onPageChange={handleChangePage}
                    onRowsPerPageChange={handleChangeRowsPerPage}
                />
            ) : null}
        </div>
    );
}
